@@ -1,18 +1,19 @@
 #Flab
-#Version 1.0.0
-#Published 20-January-2021
+#Version 1.1.0
+#Published 7-April-2022
 #Distributed under GNU GPL v3
-#Author: Nicholas Jose
+#Author: Nicholas A. Jose
 
 from flab import TaskManager, UiManager, DeviceManager
 import time
 import os
+import sys
 
 class Flab(DeviceManager.DeviceManager, TaskManager.TaskManager, UiManager.UiManager):
     #Flab inherits methods from DeviceManager, TaskManager and UiManager libraries
 
     description = 'The Flab space for shared devices, tasks and user interfaces (UIs)'
-    version = '1.0.0'
+    version = '1.1.0'
     devices = {} #device dictionary
     tasks = {}  #task dictionary
     vars = {} #variable dictionary
@@ -66,20 +67,18 @@ class Flab(DeviceManager.DeviceManager, TaskManager.TaskManager, UiManager.UiMan
             #stop all running tasks
             print('stopping running tasks')
             self.stop_all_tasks()
-            time.sleep(5)
-
+            while self.get_running_task_names() != []:
+                time.sleep(1)
             self.is_running = False
-
             print('closing flab process')
             self.flab_queue.put('close')
             print('closing ui process')
             self.ui_queue.put('close')
-
         except Exception as e:
             self.display('error in closing')
             self.display(e)
         finally:
-            pass
+            return 0
 
     #Closing of queues
     def close_queues(self):
