@@ -1,8 +1,8 @@
 #TaskManager
-#Version 0.0.5
-#Published 1-January-2021
+#Version 1.1.0
+#Published 7-April-2022
 #Distributed under GNU GPL v3
-#Author: Nicholas Jose
+#Author: Nicholas A. Jose
 
 import glob
 import threading
@@ -17,7 +17,7 @@ class TaskManager():
 
     description = 'Methods for loading, running and terminating tasks implementing python\'s threading ' \
                   'library, PyQT\'s QThreading library, asyncio, and multiprocessing'
-    version = '0.0.9'
+    version = '1.1.0'
     tasks = {} #dictionary of loaded tasks
     running_tasks = {} #dictionary of running tasks (i.e. instances of threads)
 
@@ -167,17 +167,26 @@ class TaskManager():
 
     #this method stops all running tasks
     def stop_all_tasks(self):
-        current_tasks = self.running_tasks
-        for t in current_tasks:
-            if t.__contains__('RUN_'):
+        try:
+            current_tasks = self.running_tasks
+            temp_tasks =[]
+            for t in current_tasks:
+                if t.__contains__('RUN_'):
+                    temp_tasks.append(t)
+            for t in temp_tasks:
                 tt = t.replace('RUN_','')
                 self.stop_task(tt)
+        except Exception as e:
+            print(e)
+            self.display(e)
+        finally:
+            pass
 
     #start a thread with the run method, with input arguments args and kwargs
     def start_thread(self, task_name, *args, **kwargs):
         try:
             task = self.tasks[task_name]
-            thr = ThreadTrace('run', 'task_name', target=task.run, args=args, kwargs=kwargs, daemon=True)
+            thr = ThreadTrace('run', task_name, self, target=task.run, args=args, kwargs=kwargs, daemon=True)
             thr.start()
             self.running_tasks.update({'RUN_' + task_name: thr})
         except Exception as e:
@@ -190,7 +199,7 @@ class TaskManager():
     def stop_thread(self, task_name, *args, **kwargs):
         try:
             task = self.tasks[task_name]
-            thr = ThreadTrace('stop', task_name, target=task.stop, args=args, kwargs=kwargs, daemon=True)
+            thr = ThreadTrace('stop', task_name, self, target=task.stop, args=args, kwargs=kwargs, daemon=True)
             thr.start()
             self.running_tasks.update({'STOP_' + task_name: thr})
             self.kill_thread('RUN_' + task_name)
@@ -219,11 +228,19 @@ class TaskManager():
             finally:
                 pass
 
+    #returns task instances
+    def get_running_task_names(self):
+        running_task_names = []
+        for i in self.running_tasks:
+            if self.running_tasks[i].is_alive() is True:
+                running_task_names.append(i)
+        return running_task_names
+
     #start a process with the run method,  with input arguments args and kwargs
     def start_process(self, task_name, *args, blocking=False):
         try:
             process_class = self.tasks[task_name]
-            process = Process(target=process_class.run, args=args)
+            process = Process(target=process_class.run, args=args, daemon=True)
             process.start()
             self.running_tasks.update({'RUN_' + task_name: process})
             if blocking:
@@ -246,6 +263,8 @@ class TaskManager():
                 procs.update({'RUN_' + t: process})
                 self.running_tasks.update({'RUN_' + t: process})
                 index = index + 1
+            for p in procs:
+                procs[p].join()
             if blocking:
                 for p in procs:
                     procs[p].join()
@@ -255,18 +274,18 @@ class TaskManager():
         finally:
             pass
 
-    #stop a process by calling the task stop method and then terminating the running task
+    #stop a process by terminating the running task, then calling the stop method of the process as another process.
     def stop_process(self, task_name, *args, blocking=True):
         try:
+            running_task_name = 'RUN_' + task_name
+            if running_task_name in self.get_running_task_names():
+                self.running_tasks[running_task_name].terminate()
             process_class = self.tasks[task_name]
-            process = Process(target=process_class.stop, args=args)
+            process = Process(target=process_class.stop, args=args, daemon=True)
             process.start()
             self.running_tasks.update({'STOP_' + task_name: process})
             if blocking:
                 process.join()
-            running_task = 'RUN_' + task_name
-            if running_task in self.running_tasks:
-                self.running_tasks['RUN_' + task_name].terminate()
         except Exception as e:
             self.display('Error in stopping process ' + task_name)
             self.display(e)
@@ -283,7 +302,7 @@ class TaskManager():
                     t = asyncio.create_task(task.run())
                     await t
                 asyncio.run(main())
-            thr = ThreadTrace('run', 'task_name', target=asyn_task, args=args, kwargs=kwargs, daemon=True)
+            thr = ThreadTrace('run', task_name, self, target=asyn_task, args=args, kwargs=kwargs, daemon=True)
             thr.start()
             self.running_tasks.update({'RUN_ATask_' + task_name: thr})
         except Exception as e:
@@ -307,7 +326,7 @@ class TaskManager():
                     for t in asyncio_tasks:
                         await t
                 asyncio.run(main())
-            thr = ThreadTrace('run', 'task_name', target=asyn_task, args=args, kwargs=kwargs, daemon=True)
+            thr = ThreadTrace('run', task_names, self, target=asyn_task, args=args, kwargs=kwargs, daemon=True)
             thr.start()
             self.running_tasks.update({'RUN_ATask_' + tot_task_name: thr})
         except Exception as e:
@@ -325,7 +344,7 @@ class TaskManager():
                     t = asyncio.create_task(task.stop())
                     await t
                 asyncio.run(main())
-            thr = ThreadTrace('run', 'task_name', target=asyn_task, args=args, kwargs=kwargs, daemon=True)
+            thr = ThreadTrace('run', task_name, self, target=asyn_task, args=args, kwargs=kwargs, daemon=True)
             thr.start()
             self.running_tasks.update({'STOP_ATask_' + task_name: thr})
             self.running_tasks['RUN_ATask_'+task_name].kill()
@@ -347,7 +366,7 @@ class TaskManager():
                         t = asyncio.create_task(task.stop())
                         await t
                     asyncio.run(main())
-                thr = ThreadTrace('run', 'task_name', target=asyn_task, args=args, kwargs=kwargs, daemon=True)
+                thr = ThreadTrace('run', task_name, self, target=asyn_task, args=args, kwargs=kwargs, daemon=True)
                 thr.start()
                 self.running_tasks.update({'STOP_ATask_' + task_name: thr})
             self.running_tasks['RUN_ATask_'+tot_task_name].kill()
@@ -401,21 +420,28 @@ class TaskManager():
 #A class for killable threads using traces
 class ThreadTrace(threading.Thread):
 
-    def __init__(self, task_method, task_name, *args, **kwargs):
+    def __init__(self, task_method, task_name, flab, *args, **kwargs):
         self.task_method = task_method
         self.task_name = task_name
+        self.flab = flab
         threading.Thread.__init__(self, *args, **kwargs)
         self.killed = False
 
     def start(self):
-        self.__run_backup = self.run
-        self.run = self.__run
-        threading.Thread.start(self)
+        try:
+            self.__run_backup = self.run
+            self.run = self.__run
+            threading.Thread.start(self)
+        except Exception as e:
+            self.flab.display(e)
 
     def __run(self):
-        sys.settrace(self.globaltrace)
-        self.__run_backup()
-        self.run = self.__run_backup
+        try:
+            sys.settrace(self.globaltrace)
+            self.__run_backup()
+            self.run = self.__run_backup
+        except Exception as e:
+            self.flab.display(e)
 
     def globaltrace(self, frame, event, arg):
         if event == 'call':
